
; ATMega32

; Version v0.2 27.09.2006 M'Soft Berlin

.include "m32def.inc"
.include "definitionen.inc"
.include "enc28j60.inc"

;****************************** Verktor-Liste ****************************

.CSEG
                rjmp    Reset
.org INT0addr
                reti
.org INT1addr
                reti
.org ICP1addr
                reti
.org OC1Aaddr
                reti
.org OC1Baddr
                reti
.org OVF1addr
                reti
.org OVF0addr
                reti
.org SPIaddr
                reti
.org URXCaddr
                reti
.org UDREaddr
                reti
.org UTXCaddr
                reti
.org ACIaddr
                reti

;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

; Hardware initialisieren

Reset:  ldi     TEMP_A,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_A
        ldi     TEMP_A,high(RAMEND)
        out     SPH,TEMP_A

; Port A *************************************************************

        ser     TEMP_A
        out     PORTA,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
        clr     TEMP_A           		    ; alle Eingnge
        out     DDRA,TEMP_A

; Port B *************************************************************

        clr		TEMP_A
        out     PORTB,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
        ldi     TEMP_A,(1<<ENC_CLK)+(1<<ENC_SI)+(1<<ENC_CS)+(1<<ENC_SS)
        out     DDRB,TEMP_A

; Port C *************************************************************

        ser     TEMP_A
        out     PORTC,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
        clr     TEMP_A             		    ; alle Eingnge
        out     DDRC,TEMP_A

; Port D *************************************************************

        ser     TEMP_A
        out     PORTD,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
        clr     TEMP_A                      ; erstmal alles Eingang 
        out     DDRD,TEMP_A

; UART ***************************************************************

        ldi     TEMP_A,high(UART_BAUD_SELECT)
        out     UBRRH,TEMP_A
        ldi     TEMP_A,low(UART_BAUD_SELECT)
        out     UBRRL,TEMP_A
        ldi     TEMP_A,(1<<TXEN)
        out     UCSRB,TEMP_A
        ldi     TEMP_A,(1<<URSEL)+(1<<UCSZ1)+(1<<UCSZ0)
        out     UCSRC,TEMP_A

; SPI ****************************************************************

        ldi     TEMP_A,(1<<MSTR)+(1<<SPE) //+(1<<SPR0)
        out     SPCR,TEMP_A                 ; SPI initialisieren
        sbi     SPSR,SPI2X					; Clock x2
        in      TEMP_A,SPSR                 ; Clear Flags

; Clock/2, SCLK low wenn idle, steigende Flanke, LSB first

; INT0 ***************************************************************
		  
;*********************************************************************				 

; Software initialisieren

		clr		ENC_CUR_BANK				; Invalid Bank	
		dec		ENC_CUR_BANK
		
		ldi		TEMP_A,MAC_ADR5
		sts		INT_MAC,TEMP_A

		ldi		TEMP_A,MAC_ADR4
		sts		INT_MAC+1,TEMP_A

		ldi		TEMP_A,MAC_ADR3
		sts		INT_MAC+2,TEMP_A

		ldi		TEMP_A,MAC_ADR2
		sts		INT_MAC+3,TEMP_A

		ldi		TEMP_A,MAC_ADR1
		sts		INT_MAC+4,TEMP_A

		ldi		TEMP_A,MAC_ADR0
		sts		INT_MAC+5,TEMP_A
		    
		ldi		TEMP_A,IP_ADR3
		sts		INT_IP,TEMP_A

		ldi		TEMP_A,IP_ADR2
		sts		INT_IP+1,TEMP_A

		ldi		TEMP_A,IP_ADR1
		sts		INT_IP+2,TEMP_A

		ldi		TEMP_A,IP_ADR0
		sts		INT_IP+3,TEMP_A
		    
;		load_p	X,TCP_SEQ
		clr		TEMP_A
		sts		LAST_SEQ_NR,TEMP_A
		sts		LAST_SEQ_NR+1,TEMP_A
		sts		LAST_SEQ_NR+2,XH
		sts		LAST_SEQ_NR+3,XL

		clr		ENC_FLAGS
		clr		TCP_STATUS

		ldi		TEMP_A,low(ENC_RX_BUF)
		sts		NXT_RX_BUF+1,TEMP_A

		ldi		TEMP_A,high(ENC_RX_BUF)
		sts		NXT_RX_BUF,TEMP_A

        rcall   enc_init           	    ; ENC initialisieren  
		 
;*********************************************************************
;  Main-Loop
;*********************************************************************

main:
		rcall	send_newline
		load_p	Z,(2*debug_reset)
		rcall	send_text
		rcall	send_newline
		rcall	send_newline

		rjmp	main_end						; Empfang freigeben

main_loop:
		ldi		ENC_CTRL,EIR
		rcall	enc_read_ctrl
		sbrs	ENC_DATA_L,EIR_PKTIF			; Paket da?
		rjmp	main_end						; nein
		
		rcall	get_rx_paket

		sbrs	ENC_FLAGS,FLAG_ETH_ARP
		rjmp	main_ip

		rcall	send_arp_reply
		rcall	rx_paket_ready
		cbr		ENC_FLAGS,(1<<FLAG_ETH_ARP)
		rjmp	main_end				

main_ip:
		sbrs	ENC_FLAGS,FLAG_ETH_IDCMP
		rjmp	main_tcp				

		rcall	send_echo_reply
		rcall	rx_paket_ready
		cbr		ENC_FLAGS,(1<<FLAG_ETH_IDCMP)
		rjmp	main_end

main_tcp:
		sbrs	ENC_FLAGS,FLAG_ETH_TCP
		rjmp	main_end				

		sbrc	ENC_FLAGS,FLAG_TCP_HTTP			; SYN 
		rjmp    send_http

		ldi		TEMP_A,0
		sts		TCP_D_LEN,TEMP_A				; Datenlnge auf 0	
		sts		TCP_D_LEN+1,TEMP_A
		rcall	send_tcp_reply

		rcall	rx_paket_ready
		cbr		ENC_FLAGS,(1<<FLAG_ETH_TCP)

main_end:
		ldi		ENC_CTRL,ECON1
		ldi		ENC_DATA_L,(1<<ECON1_RXEN)		; Empfnger ein	
		rcall	enc_write_ctrl

		rjmp	main_loop				

send_http:
		load_p	Z,(2*debug_http_header)
		rcall	send_text
		rcall	send_newline

		load_p	X,ENC_BUF
		load_p	Y,80
		rcall	debug_32bit_tab
		rcall	send_newline

		rcall	rx_paket_ready
		cbr		ENC_FLAGS,(1<<FLAG_ETH_TCP)
		cbr		ENC_FLAGS,(1<<FLAG_TCP_HTTP)

		rjmp	main_end

.include "enc28j60.asm"
.include "uart.asm"
.include "subroutinen.asm"
.include "network.asm"
.include "debug.asm"

;***************************************************************************
;  Programm Ende 
;***************************************************************************

.include "tabellen.inc"
