; Definitionen

; Belegungen der Atmel-Ports:

;*****************************************************************

;    PA0-PA7        frei

;    PC0-PC7        frei

;    PB0        T0        
;    PB1        T1        
;    PB2        AIN0      
;    PB3        AIN1      ENC_CS
;    PB4        SS        
;    PB5        MOSI      ENC-SI
;    PB6        MISO      ENC-SO
;    PB7        SCK       ENC-Clock


;    PD0        RxD		  RS-232       
;    PD1        TxD       RS-232
;    PD2        INT0      
;    PD3        INT1      
;    PD4        T0        
;    PD5        T1        
;    PD6        WR        
;    PD7        RD        


;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 16000000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ    UART_BAUD_RATE  = 38400
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1

;--- Flag-Werte ------------------------------------------------

.equ	FLAG_ETH_ARP	= 0
.equ	FLAG_ETH_IDCMP	= 1
.equ	FLAG_ETH_TCP	= 2
.equ	FLAG_TCP_HTTP	= 4
     
;--- Ethernet-Werte --------------------------------------------

.equ	IP_ADR3			= 192
.equ	IP_ADR2			= 168
.equ	IP_ADR1			= 0
.equ	IP_ADR0			= 100

.equ	MAC_ADR5		= $00						; MAC-Adress MSB
.equ	MAC_ADR4		= $04						; Microchip...
.equ	MAC_ADR3		= $A3 
.equ	MAC_ADR2		= $55 
.equ	MAC_ADR1		= $1C 
.equ	MAC_ADR0		= $C2						; MAC-Adress LSB

.equ	HTTP_PORT		= 80

;.equ	TCP_SEQ			= $300

;--- Daten -----------------------------------------------------

.equ	ENC_RX_BUF		= $0000
.equ	ENC_RX_BUF_LEN	= $1000

.equ	ENC_TX_BUF		= $1000

.equ	ENC_MAX_FRLEN	= 1518

.equ	ETH_TYP_ARP		= $0806
.equ	ETH_TYP_IP		= $0800

.equ	IP_TYP_ICMP		= 1

.equ	IP_TYP_TCP		= 6

.equ	ICMP_PING		= 8

.equ	TCP_FLAG_FIN	= 0
.equ	TCP_FLAG_SYN	= 1
.equ	TCP_FLAG_RST	= 2
.equ	TCP_FLAG_PSH	= 3
.equ	TCP_FLAG_ACK	= 4
.equ	TCP_FLAG_URG	= 5

.equ	TCP_STAT_OFF	= 0			; keine Connection
.equ	TCP_STAT_SYN	= 1			; SYN empfangen und quittiert
.equ	TCP_STAT_ACK	= 2			; ACK empfangen - connect
.equ	TCP_STAT_FIN	= 3			; FIN gesendet

.equ	IP_LEN			= 4			; Anzahl Bytes fr IP
.equ	MAC_LEN			= 6			; Anzahl Bytes fr MAC
.equ	ETH_HEAD_LEN	= 14		; Lnge Ethernet-Header
.equ	PSEUDO_HEAD_LEN	= 12		; Lnge TCP-Pseudo-Header

;**************************** Ram-Reservierungen ******************

;--- Internes Ram -----------------------------------------------

.DSEG

.ORG    $060    

INT_MAC:	.byte	6
INT_IP:		.byte	4

IP_H_LEN:	.byte	1
IP_D_LEN:	.byte	2		; Lnge IP-Header + Data-Payload IP
TCP_H_LEN:	.byte	1
TCP_D_LEN:	.byte	2		; Lnge der TCP-Daten nach dem Header

LAST_SEQ_NR:.byte	4		; letzte Sequenznummer
	
NXT_RX_BUF:	.byte	2

TX_CTRL:	.byte	1		; Control-Byte mu vor Ethernet-Header stehen!
; Ethernet-Header
ETH_HEADER:
ETH_D_MAC:	.byte	6
ETH_S_MAC:	.byte	6
ETH_TYPLEN:	.byte	2

; ARP-Table - 1 Eintrag
ARP_TAB_MAC:.byte	6
ARP_TAB_IP:	.byte	4

ARP_HEADER:
ARP_HWTYP:	.byte	2
ARP_PRTYP:	.byte	2
ARP_HWSIZE:	.byte	1
ARP_PRSIZE:	.byte	1
ARP_OPCODE:	.byte	2
ARP_S_MAC:	.byte	6
ARP_S_IP:	.byte	4
ARP_D_MAC:	.byte	6
ARP_D_IP:	.byte	4

IP_HEADER:
IP_VER_LEN:	.byte	1		; oberes Nibble Version, unteres Lnge in 32Bit-Worten
IP_TOS:		.byte	1		; Typ of Service
IP_TOT_LEN:	.byte	2		; komplette Lnge
IP_IDENT:	.byte	2
IP_FL_OFFS:	.byte	2		; 3 MSB Flags, 13 Bit Fragment-Offset
IP_TIME:	.byte	1
IP_PROT:	.byte	1
IP_H_CHK:	.byte	2		; Header-Checksum
IP_S_IP:	.byte	4		; Quell-IP
IP_D_IP:	.byte	4		; Ziel-IP

ICMP_HEADER:
ICMP_TYP:	.byte	1
ICMP_MODE:	.byte	1
ICMP_CHK:	.byte	2		; ICMP-Checksum	
ICMP_IDENT:	.byte	2
ICMP_SEQ:	.byte	2

TCP_PSEUDO:
TCP_S_PSEU:	.byte	4		; Quell-IP
TCP_D_PSEU:	.byte	4		; Ziel-IP
TCP_N_PSEU:	.byte	1		; 0
TCP_P_PSEU:	.byte	1		; Protokoll (IP)	
TCP_L_PSEU:	.byte	2		; Lnge TCP-Header + Daten (ohne Pseudoheader!)

TCP_HEADER:
TCP_S_PORT:	.byte	2		; Quell-Port
TCP_D_PORT:	.byte	2		; Ziel-Port
TCP_SEQ_NR:	.byte	4		; Sequenznummer
TCP_ACK_NR:	.byte	4		; Acknowledge-Nummer
TCP_FL_OFF:	.byte	2		
TCP_WINDOW:	.byte	2		
TCP_CHKSUM:	.byte	2
TCP_U_P:	.byte	2
TCP_OPTION:	.byte	16

.ORG	$140
ENC_BUF:	.byte	512

; $0380-$045F Stack                          ; 224 Byte...

;**************************** Register-Bits **************************

;--- ENC --------------------------------------------------------------

.equ    ENC_DIR      = DDRB
.equ    ENC_PORT     = PORTB                 ; MMC/SD-Card
.equ    ENC_CS       = PB3                   ; Bit fr Enable
.equ    ENC_SS       = PB4                   ; SS Ausgang fr SPI-Master
.equ    ENC_SI       = PB5                   ; Daten-In MMC-Card
.equ    ENC_SO	     = PB6                   ; Daten-Out MMC-Card
.equ    ENC_CLK      = PB7                   ; Clock MMC-Card

;--- lower Register -----------------------------------------------

.def	ENC_CUR_BANK = r2

.def	TCP_STATUS	 = r3
	
.def	TEMP_0		 = r4	
.def	TEMP_1		 = r5	
.def	TEMP_2		 = r6	
.def	TEMP_3		 = r7	

;--- higher Register ----------------------------------------------

.def    TEMP_A      = r16                   ; temporres  8Bit-Register
.def    TEMP_B      = r17 
.def    TEMP_C      = r18 
.def    TEMP_D      = r19 

.def    ENC_CTRL    = r22                   ; Adresse enc
.def    ENC_DATA_L  = r23                   ; Daten L enc
.def    ENC_DATA_H  = r24                   ; Daten H enc

.def	ENC_FLAGS	= r25

; r26/r27, r28/r29, r30/r31 fr X,Y,Z reserviert 

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro

.macro	load_seqnr					; ldt 32Bit aus @0 in TEMP_0...3
		lds		TEMP_0,@0
		lds		TEMP_1,@0+1
		lds		TEMP_2,@0+2
		lds		TEMP_3,@0+3
.endmacro

.macro	save_seqnr					; speicher 32Bit nach @0 aus TEMP_0...3
		sts		@0,TEMP_0
		sts		@0+1,TEMP_1
		sts		@0+2,TEMP_2
		sts		@0+3,TEMP_3
.endmacro
