;******************************* Sub-ENC ***********************************
; Sub: schreibt Memory-Daten
; Parameter: X enthlt Anzahl. Z Bufferadresse 
; Return:      -
; Scratch-Reg: -
;***************************************************************************
enc_write_mem:
		ldi		ENC_CTRL,WRITE_BUF_MEM

		cbi		ENC_PORT,ENC_CS			; Chip Select L

		rcall	enc_read_write			; Kommando raus

enc_write_mem_loop:
		ld		ENC_CTRL,Z+
		rcall	enc_read_write			; Daten raus

		sbiw	X,1						; Zhler
		brne	enc_write_mem_loop
		
		sbi		ENC_PORT,ENC_CS			; Chip Select H

		ret			

;******************************* Sub-ENC ***********************************
; Sub: liest Memory-Daten
; Parameter: X enthlt Anzahl. Z Bufferadresse 
; Return: 	   -   
; Scratch-Reg: -
;***************************************************************************
enc_read_mem:
		ldi		ENC_CTRL,READ_BUF_MEM

		cbi		ENC_PORT,ENC_CS			; Chip Select L

		rcall	enc_read_write			; Kommando raus

enc_read_mem_loop:
		rcall	enc_read_write
		st		Z+,ENC_CTRL				; speichern

		sbiw	X,1						; Zhler
		brne	enc_read_mem_loop
		
		sbi		ENC_PORT,ENC_CS			; Chip Select H

		ret			

;******************************* Sub-ENC ***********************************
; Sub: liest Control-Register
; Parameter: ENC_CTRL   - Adresse 
; Return:    ENC_DATA_L - Wert
; Scratch-Reg: -
;***************************************************************************
enc_read_ctrl:
		push	ENC_CTRL
		rcall	enc_set_bank			; neue Bank

		andi	ENC_CTRL,ADR_MASK
		ori		ENC_CTRL,READ_CTRL_REG

enc_read_ctrl_start:
		cbi		ENC_PORT,ENC_CS			; Chip Select L

		rcall	enc_read_write			; Kommand senden

		rcall	enc_read_write			; Daten holen
		mov		ENC_DATA_L,ENC_CTRL		; Wert nach Data

		pop		ENC_CTRL
		sbrs	ENC_CTRL,FLAG_MAC_MI	; MAC_MI - dummy-Byte verwerfen
		rjmp	enc_read_ctrl_end

		rcall	enc_read_write			; 
		mov		ENC_DATA_L,ENC_CTRL		; Wert nach Data

enc_read_ctrl_end:
		sbi		ENC_PORT,ENC_CS			; Chip Select H

		ret

;******************************* Sub-ENC ***********************************
; Sub: lscht Bit in Control-Register
; Parameter:  ENC_CTRL   - Wert
; Return:     ENC_DATA_L - Bit-Maske 
; Scratch-Reg: -
;***************************************************************************
enc_clr_bit:
		rcall	enc_set_bank			; neue Bank

enc_clr_bit_start:
		andi	ENC_CTRL,ADR_MASK
		ori		ENC_CTRL,BIT_FIELD_CLR

		rjmp	enc_write_ctrl_start

;******************************* Sub-ENC ***********************************
; Sub: setzt Bit in Control-Register
; Parameter:  ENC_CTRL   - Wert
; Return:     ENC_DATA_L - Bit-Maske
; Scratch-Reg: -
;***************************************************************************
enc_set_bit:
		rcall	enc_set_bank			; neue Bank

enc_set_bit_start:
		andi	ENC_CTRL,ADR_MASK
		ori		ENC_CTRL,BIT_FIELD_SET

		rjmp	enc_write_ctrl_start

;******************************* Sub-ENC ***********************************
; Sub: schreibt Control-Register Daten
; Parameter:  ENC_CTRL - Wert
; Return:      -
; Scratch-Reg: -
;***************************************************************************
enc_write_ctrl:
		rcall	enc_set_bank			; neue Bank

		andi	ENC_CTRL,ADR_MASK
		ori		ENC_CTRL,WRITE_CTRL_REG

enc_write_ctrl_start:
		cbi		ENC_PORT,ENC_CS			; Chip Select L

		rcall	enc_read_write

		mov    ENC_CTRL,ENC_DATA_L		; Daten raus

		rcall	enc_read_write

		sbi		ENC_PORT,ENC_CS			; Chip Select H
        ret

;******************************* Sub-ENC ***********************************
; Sub: schreibt Control-Register-Command
; Parameter:  ENC_CTRL - Wert
; Return:      -
; Scratch-Reg: -
;***************************************************************************
enc_write_cmd:
		rcall	enc_set_bank			; neue Bank

		andi	ENC_CTRL,ADR_MASK
		ori		ENC_CTRL,WRITE_CTRL_REG

		cbi		ENC_PORT,ENC_CS			; Chip Select L

		rcall	enc_read_write

		sbi		ENC_PORT,ENC_CS			; Chip Select H
        ret

;******************************* Sub-ENC ***********************************
; Sub: Setzt Bankselect in ECON1
; Parameter: Bank in Bit 6,5 von ENC_CTRL  
; Return:      -
; Scratch-Reg: -
;***************************************************************************
enc_set_bank:
		ldi		TEMP_A,ADR_MASK
		and		TEMP_A,ENC_CTRL				; nur Adresse holen
		cpi		TEMP_A,EIE					; erstes gemeinsames Register
		brge	enc_set_bank_end			; gleich/grer, also ok

		ldi		TEMP_A,BANK_MASK
		and		TEMP_A,ENC_CTRL				; nur Bank holen
		cpse	TEMP_A,ENC_CUR_BANK			; gleiche Bank?
		rjmp	enc_set_bank_start			; nein

enc_set_bank_end:		
		ret									; gleiche Bank oder gemeinsame Register

enc_set_bank_start:
		mov		ENC_CUR_BANK,TEMP_A			; Bank merken
		swap	TEMP_A						; nun Bank in Bit1+2
		lsr		TEMP_A						; nach 0+1 schieben
		push	ENC_CTRL
		push	ENC_DATA_L
		ldi		ENC_DATA_L,(1<<ECON1_BSEL0)+(1<<ECON1_BSEL1); Bitmaske in ECON1 fr BSEL0/1
		ldi		ENC_CTRL,ECON1				; Adresse

		rcall	enc_clr_bit_start			; Bank-Bits lschen

		mov		ENC_DATA_L,TEMP_A			; neue Bank
		ldi		ENC_CTRL,ECON1				; Adresse

		rcall	enc_set_bit_start			; Bank-Bits setzen

		pop		ENC_DATA_L
		pop		ENC_CTRL

        ret

;******************************* Sub-ENC ***********************************
; Sub: Liest PHY-Register
; Parameter: ENC_CTRL   - Adresse  
; Return:    ENC_DATA_L - Low-Daten
; Return:    ENC_DATA_H - High-Daten
; Scratch-Reg: -
;***************************************************************************
enc_read_phy:
		mov		ENC_DATA_L,ENC_CTRL		; PHY-Adresse nach ENC_DATA
		ldi		ENC_CTRL,MIREGADR
		rcall	enc_write_ctrl			; PHY-Adresse setzen

		ldi		ENC_DATA_L,(1<<MICMD_MIIRD)		; Lesen setzen
		ldi		ENC_CTRL,MICMD
		rcall	enc_write_ctrl			; Lesen setzen

enc_read_phy_loop:
		ldi		ENC_CTRL,MISTAT
		rcall	enc_read_ctrl			; PHY-Adresse setzen
		andi	ENC_DATA_L,(1<<MISTAT_BUSY) ; noch Busy ?
		brne	enc_read_phy_loop

		ldi		ENC_DATA_L,0			; MICMD_MIIRD lschen
		ldi		ENC_CTRL,MICMD
		rcall	enc_write_ctrl			; Lesen lschen

		ldi		ENC_CTRL,MIRDH
		rcall	enc_read_ctrl			; PHY-Daten holen

		mov		ENC_DATA_H,ENC_DATA_L

		ldi		ENC_CTRL,MIRDL
		rcall	enc_read_ctrl			; PHY-Daten holen

		ret

;******************************* Sub-ENC ***********************************
; Sub: Schreibt PHY-Register
; Parameter: ENC_CTRL   - Adresse  
; Parameter: ENC_DATA_L - Low-Daten
; Parameter: ENC_DATA_H - High-Daten
; Scratch-Reg: -
;***************************************************************************
enc_write_phy:
		push	ENC_DATA_L	
		mov		ENC_DATA_L,ENC_CTRL		; PHY-Adresse nach ENC_DATA
		ldi		ENC_CTRL,MIREGADR
		rcall	enc_write_ctrl			; PHY-Adresse setzen

		pop		ENC_DATA_L
		ldi		ENC_CTRL,MIWRL
		rcall	enc_write_ctrl			; PHY-Adresse L setzen

		mov		ENC_DATA_L,ENC_DATA_H
		ldi		ENC_CTRL,MIWRH
		rcall	enc_write_ctrl			; PHY-Adresse H setzen

enc_write_phy_loop:
		ldi		ENC_CTRL,MISTAT
		rcall	enc_read_ctrl			; PHY-Adresse setzen
		andi	ENC_DATA_L,(1<<MISTAT_BUSY) ; noch Busy ?
		brne	enc_write_phy_loop

		ret
										
;******************************* Sub-ENC ***********************************
; Sub: Schreibt und liest ein Byte
; Parameter: ENC_CTRL - Daten  
; Return:    ENC_CTRL - Daten
; Scratch-Reg: -
;***************************************************************************
enc_read_write:
		out     SPDR,ENC_CTRL			; Daten raus

enc_read_write_loop:
        sbis    SPSR,SPIF
        rjmp    enc_read_write_loop		; warten, bis gesendet

		in		ENC_CTRL,SPDR			; Daten rein
		ret

;******************************* Sub-ENC ***********************************
; Sub: Initialisiert den ENC
; Parameter:   -  
; Return:      -
; Scratch-Reg: -
;***************************************************************************
enc_init:
		rcall	wait_1s

		ldi		ENC_CTRL,SOFT_RESET				
		rcall	enc_write_cmd					; Softreset

		rcall	wait_1s

		load_p	Z,(2*enc_init_tab)

enc_init_loop:
		lpm		ENC_CTRL,Z+
		cpi		ENC_CTRL,$FF
		breq	enc_init_end

		lpm		ENC_DATA_L,Z+
		rcall	enc_write_ctrl
		rjmp	enc_init_loop

enc_init_end:

		ret

