// ENC28J60 Control Registers
// Control register definitions are a combination of address,
// bank number, and Ethernet/MAC/PHY indicator bits.
// - Register address	(bits 0-4)
// - Bank number		(bits 5-6)
// - MAC/PHY indicator	(bit 7)

.equ BANK_0		=	0b00000000
.equ BANK_1		=	0b00100000
.equ BANK_2		=	0b01000000
.equ BANK_3		=	0b01100000

.equ BANK_MASK	=	0b01100000

.equ ADR_MASK	=	0b00011111

.equ MAC_MI		=	0b10000000

.equ FLAG_MAC_MI	=	7

// All-bank registers
.equ EIE			=	$1B
.equ EIR			=	$1C
.equ ESTAT			=	$1D
.equ ECON2			=	$1E
.equ ECON1			=	$1F

// Bank 0 registers
.equ ERDPTL			=	$00+BANK_0
.equ ERDPTH			=	$01+BANK_0
.equ EWRPTL			=	$02+BANK_0
.equ EWRPTH			=	$03+BANK_0
.equ ETXSTL			=	$04+BANK_0
.equ ETXSTH			=	$05+BANK_0
.equ ETXNDL			=	$06+BANK_0
.equ ETXNDH			=	$07+BANK_0
.equ ERXSTL			=	$08+BANK_0
.equ ERXSTH			=	$09+BANK_0
.equ ERXNDL			=	$0A+BANK_0
.equ ERXNDH			=	$0B+BANK_0
.equ ERXRDPTL		=	$0C+BANK_0
.equ ERXRDPTH		=	$0D+BANK_0
.equ ERXWRPTL		=	$0E+BANK_0
.equ ERXWRPTH		=	$0F+BANK_0
.equ EDMASTL		=	$10+BANK_0
.equ EDMASTH		=	$11+BANK_0
.equ EDMANDL		=	$12+BANK_0
.equ EDMANDH		=	$13+BANK_0
.equ EDMADSTL		=	$14+BANK_0
.equ EDMADSTH		=	$15+BANK_0
.equ EDMACSL		=	$16+BANK_0
.equ EDMACSH		=	$17+BANK_0

// Bank 1 registers
.equ EHT0			=	$00+BANK_1
.equ EHT1			=	$01+BANK_1
.equ EHT2			=	$02+BANK_1
.equ EHT3			=	$03+BANK_1
.equ EHT4			=	$04+BANK_1
.equ EHT5			=	$05+BANK_1
.equ EHT6			=	$06+BANK_1
.equ EHT7			=	$07+BANK_1
.equ EPMM0			=	$08+BANK_1
.equ EPMM1			=	$09+BANK_1
.equ EPMM2			=	$0A+BANK_1
.equ EPMM3			=	$0B+BANK_1
.equ EPMM4			=	$0C+BANK_1
.equ EPMM5			=	$0D+BANK_1
.equ EPMM6			=	$0E+BANK_1
.equ EPMM7			=	$0F+BANK_1
.equ EPMCSL			=	$10+BANK_1
.equ EPMCSH			=	$11+BANK_1
.equ EPMOL			=	$12+BANK_1
.equ EPMOH			=	$13+BANK_1
.equ EWOLIE			=	$14+BANK_1
.equ EWOLIR			=	$15+BANK_1
.equ ERXFCON		=	$16+BANK_1
.equ EPKTCNT		=	$17+BANK_1

// Bank 2 registers
.equ MACON1			=	$00+BANK_2+MAC_MI
.equ MACON2			=	$01+BANK_2+MAC_MI
.equ MACON3			=	$02+BANK_2+MAC_MI
.equ MACON4			=	$03+BANK_2+MAC_MI
.equ MABBIPG		=	$04+BANK_2+MAC_MI
.equ MAIPGL			=	$06+BANK_2+MAC_MI
.equ MAIPGH			=	$07+BANK_2+MAC_MI
.equ MACLCON1		=	$08+BANK_2+MAC_MI
.equ MACLCON2		=	$09+BANK_2+MAC_MI
.equ MAMXFLL		=	$0A+BANK_2+MAC_MI
.equ MAMXFLH		=	$0B+BANK_2+MAC_MI
.equ MAPHSUP		=	$0D+BANK_2+MAC_MI
.equ MICON			=	$11+BANK_2+MAC_MI
.equ MICMD			=	$12+BANK_2+MAC_MI
.equ MIREGADR		=	$14+BANK_2+MAC_MI
.equ MIWRL			=	$16+BANK_2+MAC_MI
.equ MIWRH			=	$17+BANK_2+MAC_MI
.equ MIRDL			=	$18+BANK_2+MAC_MI
.equ MIRDH			=	$19+BANK_2+MAC_MI

// Bank 3 registers
.equ MAADR5			=	$00+BANK_3+MAC_MI
.equ MAADR6			=	$01+BANK_3+MAC_MI
.equ MAADR3			=	$02+BANK_3+MAC_MI
.equ MAADR4			=	$03+BANK_3+MAC_MI
.equ MAADR1			=	$04+BANK_3+MAC_MI
.equ MAADR2			=	$05+BANK_3+MAC_MI
.equ EBSTSD			=	$06+BANK_3
.equ EBSTCON		=	$07+BANK_3
.equ EBSTCSL		=	$08+BANK_3
.equ EBSTCSH		=	$09+BANK_3
.equ MISTAT			=	$0A+BANK_3+MAC_MI
.equ EREVID			=	$12+BANK_3
.equ ECOCON			=	$15+BANK_3
.equ EFLOCON		=	$17+BANK_3
.equ EPAUSL			=	$18+BANK_3
.equ EPAUSH			=	$19+BANK_3

// PHY registers
.equ PHCON1			=	$00
.equ PHSTAT1		=	$01
.equ PHHID1			=	$02
.equ PHHID2			=	$03
.equ PHCON2			=	$10
.equ PHSTAT2		=	$11
.equ PHIE			=	$12
.equ PHIR			=	$13
.equ PHLCON			=	$14

// ENC28J60 ERXFCON Register Bit Definition
.equ ERXFCON_UCEN	=	7
.equ ERXFCON_ANDOR	=	6
.equ ERXFCON_CRCEN	=	5
.equ ERXFCON_PMEN	=	4
.equ ERXFCON_MPEN	=	3
.equ ERXFCON_HTEN	=	2
.equ ERXFCON_MCEN	=	1
.equ ERXFCON_BCEN	=	0

// ENC28J60 EIE Register Bit Definitions
.equ EIE_INTIE		=	7
.equ EIE_PKTIE		=	6
.equ EIE_DMAIE		=	5
.equ EIE_LINKIE		=	4
.equ EIE_TXIE		=	3
.equ EIE_WOLIE		=	2
.equ EIE_TXERIE		=	1
.equ EIE_RXERIE		=	0

// ENC28J60 EIR Register Bit Definitions
.equ EIR_PKTIF		=	6
.equ EIR_DMAIF		=	5
.equ EIR_LINKIF		=	4
.equ EIR_TXIF		=	3
.equ EIR_WOLIF		=	2
.equ EIR_TXERIF		=	1
.equ EIR_RXERIF		=	0

// ENC28J60 ESTAT Register Bit Definitions
.equ ESTAT_INT		=	7
.equ ESTAT_LATECOL	=	4
.equ ESTAT_RXBUSY	=	2
.equ ESTAT_TXABRT	=	1
.equ ESTAT_CLKRDY	=	0

// ENC28J60 ECON2 Register Bit Definitions
.equ ECON2_AUTOINC	=	7
.equ ECON2_PKTDEC	=	6
.equ ECON2_PWRSV	=	5
.equ ECON2_VRPS		=	3

// ENC28J60 ECON1 Register Bit Definitions
.equ ECON1_TXRST	=	7
.equ ECON1_RXRST	=	6
.equ ECON1_DMAST	=	5
.equ ECON1_CSUMEN	=	4
.equ ECON1_TXRTS	=	3
.equ ECON1_RXEN		=	2
.equ ECON1_BSEL1	=	1
.equ ECON1_BSEL0	=	0

// ENC28J60 MACON1 Register Bit Definitions
.equ MACON1_LOOPBK	=	4
.equ MACON1_TXPAUS	=	3
.equ MACON1_RXPAUS	=	2
.equ MACON1_PASSALL	=	1
.equ MACON1_MARXEN	=	0

// ENC28J60 MACON2 Register Bit Definitions
.equ MACON2_MARST	=	7
.equ MACON2_RNDRST	=	6
.equ MACON2_MARXRST	=	3
.equ MACON2_RFUNRST	=	2
.equ MACON2_MATXRST	=	1
.equ MACON2_TFUNRST	=	0

// ENC28J60 MACON3 Register Bit Definitions
.equ MACON3_PADCFG2	=	7
.equ MACON3_PADCFG1	=	6
.equ MACON3_PADCFG0	=	5
.equ MACON3_TXCRCEN	=	4
.equ MACON3_PHDRLEN	=	3
.equ MACON3_HFRMLEN	=	2
.equ MACON3_FRMLNEN	=	1
.equ MACON3_FULDPX	=	0

// ENC28J60 MACON3 Register Bit Definitions
.equ MACON4_DEFER	=	6
.equ MACON4_BPEN	=	5
.equ MACON4_NOBKOFF	=	4

// ENC28J60 MICMD Register Bit Definitions
.equ MICMD_MIISCAN	=	1
.equ MICMD_MIIRD	=	0

// ENC28J60 MISTAT Register Bit Definitions
.equ MISTAT_NVALID	=	2
.equ MISTAT_SCAN	=	1
.equ MISTAT_BUSY	=	0

// ENC28J60 PHY PHCON1 Register Bit Definitions
.equ PHCON1_PRST	=	$8000
.equ PHCON1_PLOOPBK	=	$4000
.equ PHCON1_PPWRSV	=	$0800
.equ PHCON1_PDPXMD	=	$0100

// ENC28J60 PHY PHSTAT1 Register Bit Definitions
.equ PHSTAT1_PFDPX	=	$1000
.equ PHSTAT1_PHDPX	=	$0800
.equ PHSTAT1_LLSTAT	=	$0004
.equ PHSTAT1_JBSTAT	=	$0002

// ENC28J60 PHY PHCON2 Register Bit Definitions
.equ PHCON2_FRCLINK	=	$4000
.equ PHCON2_TXDIS	=	$2000
.equ PHCON2_JABBER	=	$0400
.equ PHCON2_HDLDIS	=	$0100

// ENC28J60 Packet Control Byte Bit Definitions
.equ PKTCTRL_PHUGEEN	=	3
.equ PKTCTRL_PPADEN		=	2
.equ PKTCTRL_PCRCEN		=	1
.equ PKTCTRL_POVERRIDE	=	0

// SPI operation codes
.equ READ_CTRL_REG		=	$00 ; OR Register-Adresse, 1 Byte Antwort 
.equ READ_BUF_MEM		=	$3A	; 1 Byte Antwort	
.equ WRITE_CTRL_REG		=	$40	; OR Register-Adresse + 1 Byte Daten
.equ WRITE_BUF_MEM		=	$7A	; 1 Byte Daten
.equ BIT_FIELD_SET		=	$80	; OR Registeradresse + Bit-Maske
.equ BIT_FIELD_CLR		=	$A0	; OR Registeradresse + Bit-Maske
.equ SOFT_RESET			=	$FF	; nichts
