;******************************** Sub-Ethernet *****************************
; Sub: Sortiert den Ehernet-Header um
; Parameter:    Typ immer IP
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
make_eth_header:
		load_p	X,ETH_S_MAC						; Quelle aus IP
		load_p	Y,ETH_D_MAC						; als Ziel in Ehernet-Header
		ldi		TEMP_A,MAC_LEN
		rcall	copy_ram

		load_p	X,INT_MAC						; eigene MAC
		load_p	Y,ETH_S_MAC						; als Quelle in Ehernet-Header
		ldi		TEMP_A,MAC_LEN
		rcall	copy_ram

		load_p	X,ETH_TYP_IP					; Typ/Len in Ehernet-Header
		st		Y+,XH
		st		Y+,XL

		ret

;******************************** Sub-Ethernet *****************************
; Sub: Sortiert den IP-Header um
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
make_ip_header:
		load_p	X,IP_S_IP						; Quell-IP holen
		load_p	Y,IP_D_IP						; als Ziel-IP in IP-Header
		ldi		TEMP_A,IP_LEN
		rcall	copy_ram

		load_p	X,INT_IP						; Meine IP holen
		load_p	Y,IP_S_IP						; als Quell-IP in IP-Header
		ldi		TEMP_A,IP_LEN
		rcall	copy_ram

		clr		ENC_DATA_L
		sts		IP_H_CHK,ENC_DATA_L				; IP-Header-Checksum auf 0
		sts		IP_H_CHK+1,ENC_DATA_L

		load_p	Z,IP_HEADER
		lds		XL,IP_VER_LEN					; Headerlnge holen
		andi	XL,$0F							; Version ausblenden
		clr		XH
		lsl		XL								; *4
		lsl		XL
		rcall	make_chksum_16
		
		sts		IP_H_CHK,XH						; Checksumme in Header
		sts		IP_H_CHK+1,XL

		ret

;******************************** Sub-Ethernet *****************************
; Sub: Erzeugt den TCP-Pseudo-Header
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
make_pseudo_header:
		load_p	X,IP_S_IP						; Quell-IP holen
		load_p	Y,TCP_S_PSEU					; in Pseudo-Header
		ldi		TEMP_A,IP_LEN
		rcall	copy_ram

		load_p	X,IP_D_IP						; Ziel-IP holen
		ldi		TEMP_A,IP_LEN
		rcall	copy_ram

		clr		TEMP_A
		st		Y+,TEMP_A						; 0-Byte rein

		ldi		TEMP_A,IP_TYP_TCP				; Typ rein
		st		Y+,TEMP_A

		lds		TEMP_B,TCP_FL_OFF				; Offset holen
		swap	TEMP_B							; Anzahl nach unten
		andi	TEMP_B,$0F
		lsl		TEMP_B
		lsl		TEMP_B							; Low-Byte Headerlnge
		clr		TEMP_A							; High-Byte Headerlnge
		
		lds		TEMP_C,TCP_D_LEN				; Lnge TCP + Daten
		lds		TEMP_D,TCP_D_LEN+1
		add		TEMP_B,TEMP_D
		add		TEMP_A,TEMP_C				
		st		Y+,TEMP_A						; Lnge in Pseudoheader
		st		Y,TEMP_B				

 		ret

;******************************** Sub-Ethernet *****************************
; Sub: Erzeugt den TCP-Header
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
make_tcp_header:
		lds		TEMP_A,TCP_S_PORT				; Quellport in Zielport
		lds		TEMP_B,TCP_S_PORT+1
		sts		TCP_D_PORT,TEMP_A
		sts		TCP_D_PORT+1,TEMP_B

		load_p	X,HTTP_PORT						; Quellport setzen
		sts		TCP_S_PORT,XH
		sts		TCP_S_PORT+1,XL

		load_seqnr	TCP_SEQ_NR					; Sequenznummer+1 als ACK

		rcall	inc_seq_nr
				
		save_seqnr	TCP_ACK_NR

		load_seqnr	LAST_SEQ_NR					; eigene Sequenznummer

		rcall	inc_seq_nr

		save_seqnr	TCP_SEQ_NR

		save_seqnr	LAST_SEQ_NR					; und merken

		clr		TEMP_A
		sts		TCP_CHKSUM,TEMP_A				; Checksumme in Header auf 0
		sts		TCP_CHKSUM+1,TEMP_A

		load_p	X,TCP_HEADER					; TCP-Header nach ENC_BUF
		lds		TEMP_A,TCP_FL_OFF				; Offset holen
		swap	TEMP_A							; Anzahl nach unten
		andi	TEMP_A,$0F
		lsl		TEMP_A
		lsl		TEMP_A
		sts		TCP_H_LEN,TEMP_A				; merken

		ret

;******************************** Sub-Ethernet *****************************
; Sub: Holt ein Header eines Paketes und wertet aus
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
get_rx_paket:
		clr		ENC_FLAGS
		lds		ENC_DATA_L,NXT_RX_BUF+1		; nchster zu lesender Buffer
		ldi		ENC_CTRL,ERDPTL
		rcall	enc_write_ctrl

		lds		ENC_DATA_L,NXT_RX_BUF
		ldi		ENC_CTRL,ERDPTH
		rcall	enc_write_ctrl

		load_p	Z,ENC_BUF					; 6 Byte Flags und Lnge nach ENC_BUF
		load_p	X,6
		rcall	enc_read_mem

		lds		TEMP_A,ENC_BUF				; neue Startadresse merken
		sts		NXT_RX_BUF+1,TEMP_A
		lds		TEMP_A,ENC_BUF+1
		sts		NXT_RX_BUF,TEMP_A

		lds		ENC_DATA_L,ENC_BUF+4		; Flags
		lds		ENC_DATA_H,ENC_BUF+5

		sbrs	ENC_DATA_L,7				; OK?
		rjmp	rx_paket_ready				; nein

		load_p	Z,ETH_D_MAC					; Ehernet-Header nach ETH_D_MAC...
		load_p	X,ETH_HEAD_LEN
		rcall	enc_read_mem

		lds		ENC_DATA_H,ETH_TYPLEN
		cpi		ENC_DATA_H,high(ETH_TYP_ARP)
		breq	get_rx_paket_arp

		rjmp	rx_paket_ready				; kein ARP, kein IP

get_rx_paket_arp:		
		lds		ENC_DATA_L,ETH_TYPLEN+1
		cpi		ENC_DATA_L,low(ETH_TYP_ARP)
		brne	get_rx_paket_ip				; kein ARP

		sbr		ENC_FLAGS,(1<<FLAG_ETH_ARP)

		load_p	Z,ARP_HEADER				; ARP-Daten einlesen.
		load_p	X,28
		rcall	enc_read_mem

		rjmp	get_rx_paket_end

get_rx_paket_ip:
		load_p	Z,IP_HEADER					; IP-Header einlesen.
		load_p	X,20						; ### Optionen fehlen!!! ###
		rcall	enc_read_mem

		load_p	Z,IP_D_IP 				    ; Ziel IP aus IP-Header
		rcall	check_ip
		brne	rx_paket_ready				; nicht meine IP gesucht

		lds		ENC_DATA_L,IP_PROT			; Protokoll
		cpi		ENC_DATA_L,IP_TYP_ICMP		; IDCMP	?	
		breq	get_rx_paket_icmp			; ja

get_rx_paket_tcp:
		cpi		ENC_DATA_L,IP_TYP_TCP		; TCP ?	
		brne	rx_paket_ready				; nein

		load_p	Z,TCP_HEADER				; TCP erste 4 Byte holen
		load_p	X,1*4						; 1. Word (Ports)
		rcall	enc_read_mem

		lds		XH,TCP_D_PORT
		lds		XL,TCP_D_PORT+1
		ldi		YH,high(HTTP_PORT)
		ldi		YL,low(HTTP_PORT)
		cp		YL,XL
		cpc		YH,XH						; richtiger Port?
		brne	rx_paket_ready				; nein

; hier TCP
		load_p	X,4*4						; 3 Words TCP-Header
		rcall	enc_read_mem

		lds		TEMP_A,TCP_FL_OFF
		swap	TEMP_A						; Lnge nach unten
		andi	TEMP_A,$0F					; nur Lnge
		subi	TEMP_A,5					; schon gelesene Words abziehen

		tst		TEMP_A						; alles da?
		breq	get_rx_paket_tcp_ok			; ja

		cpi		TEMP_A,3					; mehr als 2*32Byie?
		brge	rx_paket_ready				; ja, wegwerfen

		lsl		TEMP_A
		lsl		TEMP_A						; *4

		mov		XL,TEMP_A
		clr		XH							; restliche Words TCP-Header
		rcall	enc_read_mem

get_rx_paket_tcp_ok:
		sbr		ENC_FLAGS,(1<<FLAG_ETH_TCP)
		rjmp	get_rx_paket_end

get_rx_paket_icmp:
		load_p	Z,ICMP_HEADER				; IDCMP erste 8 Byte holen
		load_p	X,8
		rcall	enc_read_mem
		
		lds		ENC_DATA_L,ICMP_TYP
		cpi		ENC_DATA_L,ICMP_PING		; Ping-Request?
		brne	rx_paket_ready				; nein

		sbr		ENC_FLAGS,(1<<FLAG_ETH_IDCMP)

get_rx_paket_end:
		ret

;******************************** Sub-Ethernet *****************************
; Sub: Gibt RX-Buffer frei, wenn Paket erledigt
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
rx_paket_ready:
		lds		ENC_DATA_L,NXT_RX_BUF+1		; Freigabezeiger setzen
		ldi		ENC_CTRL,ERXRDPTL						
		rcall	enc_write_ctrl

		lds		ENC_DATA_L,NXT_RX_BUF
		ldi		ENC_CTRL,ERXRDPTH						
		rcall	enc_write_ctrl

		ldi		ENC_CTRL,ECON2					; Anzahl Pakete im Buffer -1
		ldi		ENC_DATA_L,(1<<ECON2_PKTDEC)	; Bit lschen durch 1 schreiben
		rcall	enc_set_bit

		ret

;******************************** Sub-Ethernet *****************************
; Sub: Sendet ein Paket
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
put_tx_paket:
		ldi		ENC_DATA_L,low(ENC_TX_BUF)		; Sendebuffer auf Anfang setzen
		ldi		ENC_CTRL,ETXSTL
		rcall	enc_write_ctrl

		ldi		ENC_DATA_L,high(ENC_TX_BUF)
		ldi		ENC_CTRL,ETXSTH
		rcall	enc_write_ctrl

		ldi		ENC_DATA_L,low(ENC_TX_BUF)		; Writepointer auf Anfang setzen
		ldi		ENC_CTRL,EWRPTL
		rcall	enc_write_ctrl

		ldi		ENC_DATA_L,high(ENC_TX_BUF)
		ldi		ENC_CTRL,EWRPTH
		rcall	enc_write_ctrl

		load_p	Z,TX_CTRL						; Zeiger auf Ethernet-Header
		ldi		ENC_DATA_L,0b00000000			; Controlwort
		st		Z,ENC_DATA_L					; Z weiter auf Start

		load_p	X,ETH_HEAD_LEN+1				; Header + Control
		rcall	enc_write_mem

		lds		XH,IP_D_LEN						; Lnge holen
		lds		XL,IP_D_LEN+1
		load_p	Z,ENC_BUF						; Zeiger auf Daten
		rcall	enc_write_mem
	
		lds		XH,IP_D_LEN						; Lnge holen
		lds		XL,IP_D_LEN+1
		ldi		ENC_DATA_L,low(ENC_TX_BUF)		; Writepointer-Start
		ldi		ENC_DATA_H,high(ENC_TX_BUF)
		adiw	X,ETH_HEAD_LEN					; + Ethernetheader
		add		XL,ENC_DATA_L					; Zum Start dazu
		adc		XH,ENC_DATA_H

		mov		ENC_DATA_L,XL					; Sendebuffer-Ende
		ldi		ENC_CTRL,ETXNDL
		rcall	enc_write_ctrl

		mov		ENC_DATA_L,XH
		ldi		ENC_CTRL,ETXNDH
		rcall	enc_write_ctrl

		ldi		ENC_CTRL,ECON1
		ldi		ENC_DATA_L,(1<<ECON1_TXRTS)
		rcall	enc_write_ctrl
		
put_tx_paket_wait:
		ldi		ENC_CTRL,ECON1
		rcall	enc_read_ctrl
		sbrc	ENC_DATA_L,ECON1_TXRTS
		rjmp	put_tx_paket_wait

; hier kann Status ausgelesen werden (7 Byte)

		ret

;******************************** Sub-Ethernet *****************************
; Sub: Sendet ARP-Reply
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
send_arp_reply:
		load_p	Z,ARP_D_IP						; gesuchte IP aus ARP
		rcall	check_ip
		brne	send_arp_reply_end				; nicht meine IP gesucht
		
		load_p	X,ARP_S_MAC						; Quelle aus ARP
		load_p	Y,ETH_D_MAC						; als Ziel in Ehernet-Header
		ldi		TEMP_A,6
		rcall	copy_ram

		load_p	X,INT_MAC						; eigene MAC
		load_p	Y,ETH_S_MAC						; als Quelle in Ehernet-Header
		ldi		TEMP_A,6
		rcall	copy_ram

		ldi		TEMP_A,high(ETH_TYP_ARP)		; Typ/Len in Ehernet-Header
		st		Y+,TEMP_A
		ldi		TEMP_A,low(ETH_TYP_ARP)
		st		Y+,TEMP_A

		load_p	X,ARP_HWTYP						; ARP-Header
		load_p	Y,ENC_BUF						; in Buffer
		ldi		TEMP_A,7
		rcall	copy_ram

		ldi		ENC_DATA_L,2					; ARP-Reply OpCode
		st		Y+,ENC_DATA_L

		load_p	X,INT_MAC
		ldi		TEMP_A,10						; Quell-MAC+IP in ARP-Header
		rcall	copy_ram

		load_p	X,ARP_S_MAC						; Ziel-MAC+IP in ARP-Header
		ldi		TEMP_A,10
		rcall	copy_ram

		load_p	X,ARP_S_MAC						; Quelle aus ARP
		load_p	Y,ARP_TAB_MAC					; in ARP-Table merken
		ldi		TEMP_A,10
		rcall	copy_ram

		clr		TEMP_A
		sts		IP_D_LEN,TEMP_A
		ldi		TEMP_A,28						; Lnge ARP
		sts		IP_D_LEN+1,TEMP_A
		rcall	put_tx_paket

send_arp_reply_end:
		ret

;******************************** Sub-Ethernet *****************************
; Sub: Sendet Echo-Reply
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
send_echo_reply:
; ETH-Header basteln
		rcall	make_eth_header

; IP-Header basteln
		rcall	make_ip_header

; ICMP-Header basteln
		load_p	Z,ICMP_HEADER
		clr		ENC_DATA_L						; Typ Echo Reply
		st		Z+,ENC_DATA_L
		st		Z+,ENC_DATA_L					; Code 0

		sts		ICMP_CHK,ENC_DATA_L				; Checksumme in Header auf 0
		sts		ICMP_CHK+1,ENC_DATA_L

		load_p	Y,ENC_BUF

		load_p	X,IP_HEADER						; IP-Header nach ENC_BUF
		ldi		TEMP_A,20
		rcall	copy_ram

		load_p	X,ICMP_HEADER					; ICMP-Header nach ENC_BUF
		ldi		TEMP_A,8
		rcall	copy_ram

		rcall	get_ip_data_len					; Datenlnge im IP-Paket holen

		push	XH								; Lnge retten
		push	XL

		sbiw	X,8								; ICMP-Ping Headerlnge abziehen			
		
		movw	Z,Y								; in Y Zeiger auf ENC_BUF
		rcall	enc_read_mem					; Ping-Daten holen
		pop		XL
		pop		XH
		push	XH
		push	XL

		load_p	Z,ENC_BUF+20					; ICMP-Header
		rcall	make_chksum_16	

		sts		ENC_BUF+22,XH					; Checksumme in Header
		sts		ENC_BUF+23,XL

		pop		XL
		pop		XH

		adiw	X,28							; 20 IP, 8 ICMP
		sts		IP_D_LEN,XH
		sts		IP_D_LEN+1,XL
		rcall	put_tx_paket

send_echo_reply_end:
		ret

;******************************** Sub-Ethernet *****************************
; Sub: Sendet TCP-Reply
; Parameter:    -
; Return:       - 
; Scratch-Reg:  -
;***************************************************************************
send_tcp_reply:
; ETH-Header basteln
		rcall	make_eth_header

; IP-Header basteln
		rcall	make_ip_header

; TCP-Header basteln
		rcall	make_tcp_header

		lds		TEMP_A,TCP_FL_OFF+1				; Flags holen
		sbrs	TEMP_A,TCP_FLAG_SYN				; SYN ?
		rjmp	send_tcp_reply_ack				; nein

		sbr		TEMP_A,(1<<TCP_FLAG_ACK)		; ACK setzen
		rjmp	send_tcp_reply_flags

send_tcp_reply_ack:
		sbrs	TEMP_A,TCP_FLAG_ACK				; ACK ?
		rjmp	send_tcp_reply_flags			; ja

		cbr		TEMP_A,(1<<TCP_FLAG_ACK)
		sbr		ENC_FLAGS,(1<<FLAG_TCP_HTTP)	; Verbindung ok, HTTP senden

send_tcp_reply_flags:
		sts		TCP_FL_OFF+1,TEMP_A				; Flags speichern

		load_p	Y,ENC_BUF+PSEUDO_HEAD_LEN
		load_p	X,TCP_HEADER					; TCP-Header in Buffer
		lds		TEMP_A,TCP_H_LEN
		rcall	copy_ram

		load_p	Z,(2*http_txt_1)				; Y zeigt auf Buffer

;		sbrc	ENC_FLAGS,FLAG_TCP_HTTP	
;		rcall	copy_http

; Pseudo-Header basteln
		rcall	make_pseudo_header

		load_p	Y,ENC_BUF
		load_p	X,TCP_PSEUDO					; Pseudoheader in Buffer
		ldi		TEMP_A,PSEUDO_HEAD_LEN
		rcall	copy_ram

		load_p	Z,ENC_BUF						; TCP-Header + Pseudoheader
		lds		XL,TCP_H_LEN
		clr		XH								; Lnge TCP
		adiw	X,PSEUDO_HEAD_LEN				; Pseudoheader-Lnge	
		rcall	make_chksum_16					; Checksumme berechnen

		sts		TCP_CHKSUM,XH					; Checksumme in TCP-Header
		sts		TCP_CHKSUM+1,XL

		load_p	Y,ENC_BUF						; Ziel
		load_p	X,IP_HEADER						; IP-Header nach ENC_BUF
		ldi		TEMP_A,20						; Lnge IP-Header
		rcall	copy_ram

		load_p	X,TCP_HEADER					; TCP-Header nach ENC_BUF
		lds		TEMP_A,TCP_H_LEN				; Lnge zurck
		rcall	copy_ram

		lds		XL,TCP_H_LEN					; Lnge zurck
		clr		XH
		adiw	X,20							; +20 IP-Haeder	
		sts		IP_D_LEN,XH
		sts		IP_D_LEN+1,XL
		rcall	put_tx_paket

send_tcp_reply_end:
		cbr		ENC_FLAGS,(1<<FLAG_TCP_HTTP)
		ret

